#ifndef ddhUndoableKnife_H
#define ddhUndoableKnife_H

#include "ac_plugin.h"
#include "Undoables.h"

//---------------------------------------------------------------------------------------
class DDHUndoableKnife : public Undoable
{

private:

   // New vertex data
   struct t_mKnifeTargetObject
   {
      ACObject *parent;
      ACObject *object;
   };

   // The object from which cutting triangles are gathered
   ACObject *mKnifeObject;
   // If TRUE, surfaces connected to new vertices are sliced
   bool mSliceSurfacesInd;
   // If TRUE, surfaces on or in "front" of the knife are cut away into a new object
   bool mCutAwayInd;
   // If true, the knife object is deleted
   bool mDeleteKnifeInd;

   // Whether the Undo was called
   bool mUndoCalled;

   // The deleted knife
   ACObject *mDeletedKnife;
   // The parent of the knife (used in case the knife is deleted)
   ACObject *mDeletedKnifeOwner;

   // Objects before the knife operation
   List *mPreKnife;
   // Objects after the knife operation
   List *mPostKnife;
   // Selected objects after the knife operation
   List *mPostSelectedObjects;

   // Inserts a new vertex into an edge where it is intersected by a triangle.
   // If the edge is intersected at one of the end points, the insert does not occur.
   // param newvertex : the vertex created on the edge
   // param edge : the edge
   // param knifeSurf : the triangle
   // returns : pointer to "newvertex"
   Vertex *DDHUndoableKnife::intersectEdgeByTriangle( Vertex *newvertex, ACEdge *edge,
                                                      Surface *knifeSurf );

   // Slice surfaces
   void DDHUndoableKnife::sliceSurfaces( List *newvertlist );

   // Determines the disposition of the vertex to the knife object.
   // If the vertex is inside/behind any triangles of the knife object,
   // returns TRUE.
   bool vertInsideKnife( Vertex *v, List *knifetris );

   // Cut away surfaces based on the knife's normals (uses vertInsideKnife).
   void cutAwaySurfaces( List *knifetris );

   // Copy surfaces into a new set of surfaces/vertices
   // WARNINGL Will assumes that "outsurf" and "outverts" are addresses to NULL pointers
   void copySurfaces( List *insurf, List **outsurf, List **outverts );

public:

   // Constructor
   DDHUndoableKnife( ACObject *ko, bool doslice, bool cutaway, bool delknife );
   // Destructor
   ~DDHUndoableKnife();

   // Add vertices where the knife intersects the selection
   void execute();

   void undo();
   void redo();

};

#endif

