#include "DDHUndoableKnife.h"
#include <stdio.h>
#include <string.h>

// Global knife object
static ACObject *ddhStaticKnifeObject = NULL;

//---------------------------------------------------------------------------------------
// Set the knife object
// Called from TCL knife interface ("Set knife") to set the object to be used as the knife.
// Sets the static "staticKnifeObject" variable to the currently selected object.
static void ddhknife_setKnife()
{

   if( ac_get_select_mode() != SELECT_GROUP && ac_get_select_mode() != SELECT_OBJECT )
   {
      message_dialog( "knife : must be in Object or Group mode" );
   }

   List *l = ac_selection_get_objects();
   if( list_count( l ) > 1 )
   {
      message_dialog( "knife : select only one object" );
      list_free( &l );
      return;
   }
   if( list_count( l ) == 0 )
   {
      message_dialog( "knife : no object selected" );
      return;
   }
   ddhStaticKnifeObject = ( ACObject * )l->data;
   list_free( &l );
   char *kname = ac_object_get_name( ddhStaticKnifeObject );
   if( kname != NULL && strlen( kname ) > 0 )
   {
      tcl_command( "set ddhknife_knife_name \"%s\"", kname );
   }
   else
   {
      tcl_command( "set ddhknife_knife_name \"Unnamed\"" );
   }

}
//---------------------------------------------------------------------------------------
// Perform a knife cut
static void ddhknife_selectedGo( int argc, const char* argv[] )
{

   if( argc < 4 )
   {
      message_dialog( "Insufficient parameters passed to knife utility; looking for <slice_surfaces_ind> <cut_away_ind> <delete_knife_ind>" );
      return;
   }

   // Get params
   bool sliceSurfacesInd = ( strcmp( argv[1], "0" ) != 0 );
   bool cutAwayInd = ( strcmp( argv[2], "0" ) != 0 );
   bool deleteKnifeInd = ( strcmp( argv[3], "0" ) != 0 );

   // Add undoable knife
   add_undoable( "knife",
                  new DDHUndoableKnife( ddhStaticKnifeObject,
                                        sliceSurfacesInd,
                                        cutAwayInd,
                                        deleteKnifeInd ));

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{

   static bool firstTime = true;
   if( firstTime )
   {

      char *path = ( char * )strdup( d->plugin_dir_path );
#ifdef WINDOWS
      ac_unbackslash_string( path );
#endif

      tcl_command( "source \"%s/knife.tcl\"", path );
      ac_add_tools_menu_item( "Knife...", "ddhknife_dialog",
                              "Cut selected objects using a knife object" );
      ac_add_command_full( "ddhknife_go", ( void * )ddhknife_selectedGo,
                            3, "argv", "<slice_surfaces_ind> <cut_away_ind> <delete_knife_ind>",
                           "Knife utility" );
      ac_add_command( "ddhknife_set", ddhknife_setKnife );

      firstTime = false;

   }

   return 0;

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
   return 0;
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
   return( "Knife - version 1.3 - Dennis Hawthorne" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
   return( "Knife utility - Slices the edges of surfaces using a knife object" );
}
//---------------------------------------------------------------------------------------

